@@ -16,11 +16,11 @@ use bevy::{
         texture::{BevyDefault, CachedTexture, TextureCache},
         RenderApp, RenderStage,
     },
+    utils::HashMap,
 };
 
 mod deferred;
 mod overlay;
-mod storage_vec;
 mod tracing;
 mod voxel;
 
@@ -49,7 +49,6 @@ pub mod draw_3d_graph {
 }
 
 pub use deferred::DeferredMaterialPlugin;
-pub use storage_vec::StorageVec;
 pub use tracing::TracingMaterialPlugin;
 pub use voxel::VoxelMaterialPlugin;
 
@@ -67,10 +66,7 @@ impl Plugin for VoxelConeTracingPlugin {
             .add_plugin(OverlayPlugin)
             .add_plugin(DeferredMaterialPlugin::<StandardMaterial>::default())
             .add_plugin(VoxelMaterialPlugin::<StandardMaterial>::default())
-            .add_plugin(TracingMaterialPlugin::<StandardMaterial>::default())
-            .add_system_to_stage(CoreStage::PostUpdate, add_volume_overlay.exclusive_system())
-            .add_system_to_stage(CoreStage::PostUpdate, add_volume_views.exclusive_system())
-            .add_system_to_stage(CoreStage::PostUpdate, check_visibility);
+            .add_plugin(TracingMaterialPlugin::<StandardMaterial>::default());
 
         let mut shaders = app.world.get_resource_mut::<Assets<Shader>>().unwrap();
         shaders.set_untracked(
@@ -104,23 +100,8 @@ impl Plugin for VoxelConeTracingPlugin {
 
         render_app
             .init_resource::<VolumeMeta>()
-            .add_system_to_stage(
-                RenderStage::Extract,
-                extract_volumes.label(VoxelConeTracingSystems::ExtractVolumes),
-            )
-            .add_system_to_stage(
-                RenderStage::Extract,
-                extract_views.label(VoxelConeTracingSystems::ExtractViews),
-            )
-            .add_system_to_stage(
-                RenderStage::Prepare,
-                prepare_volumes.label(VoxelConeTracingSystems::PrepareVolumes),
-            )
-            .add_system_to_stage(
-                RenderStage::Queue,
-                queue_volume_view_bind_groups
-                    .label(VoxelConeTracingSystems::QueueVolumeViewBindGroups),
-            );
+            .add_system_to_stage(RenderStage::Extract, extract_volumes)
+            .add_system_to_stage(RenderStage::Prepare, prepare_volumes);
 
         let mut render_graph = render_app.world.get_resource_mut::<RenderGraph>().unwrap();
 
@@ -214,21 +195,6 @@ impl Plugin for VoxelConeTracingPlugin {
     }
 }
 
-#[derive(Debug, Hash, PartialEq, Eq, Clone, SystemLabel)]
-pub enum VoxelConeTracingSystems {
-    ExtractVolumes,
-    ExtractViews,
-    ExtractReceiverFilter,
-    PrepareVolumes,
-    QueueVolumeViewBindGroups,
-    QueueVoxelBindGroups,
-    QueueVoxel,
-    QueueMipmapBindGroups,
-    QueueTracing,
-    QueueTracingBindGroups,
-    QueueDeferred,
-}
-
 /// Marker component for meshes not casting GI.
 #[derive(Component)]
 pub struct NotGiCaster;
@@ -261,30 +227,11 @@ impl Default for Volume {
     }
 }
 
-#[derive(Component, Clone)]
-pub struct VolumeOverlay {
-    pub irradiance_size: Extent3d,
-    pub irradiance: Handle<Image>,
-    pub irradiance_resolve: Handle<Image>,
-
-    pub albedo_size: Extent3d,
-    pub albedo: Handle<Image>,
-    pub albedo_resolve: Handle<Image>,
-}
-
-#[derive(Component)]
-pub struct VolumeView;
-
 #[derive(Component, Clone)]
 pub struct VolumeUniformOffset {
     pub offset: u32,
 }
 
-#[derive(Component, Clone)]
-pub struct VoxelBufferOffset {
-    pub offset: u32,
-}
-
 #[derive(Component)]
 pub struct VolumeColorAttachment {
     pub texture: CachedTexture,
@@ -292,8 +239,6 @@ pub struct VolumeColorAttachment {
 
 #[derive(Component)]
 pub struct VolumeBindings {
-    pub irradiance_depth_texture: CachedTexture,
-    pub albedo_depth_texture: CachedTexture,
     pub voxel_texture: CachedTexture,
     pub anisotropic_textures: Vec<CachedTexture>,
     pub texture_sampler: Sampler,
@@ -305,123 +250,35 @@ pub struct GpuVolume {
     max: Vec3,
 }
 
-#[derive(Clone, AsStd140)]
-pub struct GpuVoxel {
-    color: [u32; 2],
-}
-
 #[derive(AsStd140)]
 pub struct GpuVoxelBuffer {
-    data: [GpuVoxel; VOXEL_COUNT],
+    data: [u32; VOXEL_COUNT],
 }
 
 #[derive(Default)]
 pub struct VolumeMeta {
     volume_uniforms: DynamicUniformVec<GpuVolume>,
-    voxel_buffers: StorageVec<GpuVoxelBuffer>,
+    voxel_buffers: HashMap<Entity, Buffer>,
 }
 
-pub fn add_volume_overlay(
-    mut commands: Commands,
-    mut meshes: ResMut<Assets<Mesh>>,
-    mut materials: ResMut<Assets<OverlayMaterial>>,
-    mut images: ResMut<Assets<Image>>,
-    msaa: Res<Msaa>,
-    windows: Res<Windows>,
-    volumes: Query<(Entity, &Volume), Without<VolumeOverlay>>,
-) {
-    if let Some(window) = windows.get_primary() {
-        let width = window.width() as u32;
-        let height = window.height() as u32;
-
-        for (entity, _) in volumes.iter() {
-            let irradiance_size = Extent3d {
-                width: width >> 1,
-                height: height >> 1,
-                depth_or_array_layers: 1,
-            };
-            let mut image = Image::new_fill(
-                irradiance_size,
-                TextureDimension::D2,
-                &[0, 0, 0, 255],
-                TextureFormat::bevy_default(),
-            );
-            image.texture_descriptor.usage = TextureUsages::COPY_DST
-                | TextureUsages::RENDER_ATTACHMENT
-                | TextureUsages::TEXTURE_BINDING;
-
-            image.texture_descriptor.sample_count = msaa.samples;
-            let irradiance = images.add(image.clone());
-
-            image.texture_descriptor.sample_count = 1;
-            let irradiance_resolve = images.add(image);
-
-            let albedo_size = Extent3d {
-                width,
-                height,
-                depth_or_array_layers: 1,
-            };
-            let mut image = Image::new_fill(
-                albedo_size,
-                TextureDimension::D2,
-                &[0, 0, 0, 255],
-                TextureFormat::bevy_default(),
-            );
-            image.texture_descriptor.usage = TextureUsages::COPY_DST
-                | TextureUsages::RENDER_ATTACHMENT
-                | TextureUsages::TEXTURE_BINDING;
-            image.sampler_descriptor.mag_filter = FilterMode::Linear;
-            image.sampler_descriptor.min_filter = FilterMode::Linear;
-
-            image.texture_descriptor.sample_count = msaa.samples;
-            let albedo = images.add(image.clone());
-
-            image.texture_descriptor.sample_count = 1;
-            let albedo_resolve = images.add(image);
-
-            commands.spawn_bundle(MaterialMeshBundle::<OverlayMaterial> {
-                mesh: meshes.add(shape::Quad::new(Vec2::ZERO).into()),
-                material: materials.add(OverlayMaterial {
-                    irradiance_image: irradiance_resolve.clone(),
-                    albedo_image: albedo_resolve.clone(),
-                }),
-                ..Default::default()
-            });
-
-            commands.entity(entity).insert(VolumeOverlay {
-                irradiance_size,
-                irradiance,
-                irradiance_resolve,
-                albedo_size,
-                albedo,
-                albedo_resolve,
-            });
-        }
-    }
-}
-
-pub fn extract_volumes(mut commands: Commands, volumes: Query<(Entity, &Volume, &VolumeOverlay)>) {
-    for (entity, volume, overlay) in volumes.iter() {
-        commands
-            .get_or_spawn(entity)
-            .insert_bundle((volume.clone(), overlay.clone()));
+fn extract_volumes(mut commands: Commands, volumes: Query<(Entity, &Volume)>) {
+    for (entity, volume) in volumes.iter() {
+        commands.get_or_spawn(entity).insert(volume.clone());
     }
 }
 
 #[allow(clippy::too_many_arguments)]
-pub fn prepare_volumes(
+fn prepare_volumes(
     mut commands: Commands,
     render_device: Res<RenderDevice>,
     render_queue: Res<RenderQueue>,
-    msaa: Res<Msaa>,
     mut texture_cache: ResMut<TextureCache>,
-    mut volumes: Query<(Entity, &Volume, &VolumeOverlay)>,
+    mut volumes: Query<(Entity, &Volume)>,
     mut volume_meta: ResMut<VolumeMeta>,
 ) {
     volume_meta.volume_uniforms.clear();
-    volume_meta.voxel_buffers.clear();
 
-    for (entity, volume, overlay) in volumes.iter_mut() {
+    for (entity, volume) in volumes.iter_mut() {
         let volume_uniform_offset = VolumeUniformOffset {
             offset: volume_meta.volume_uniforms.push(GpuVolume {
                 min: volume.min,
@@ -429,9 +286,16 @@ pub fn prepare_volumes(
             }),
         };
 
-        let voxel_buffer_offset = VoxelBufferOffset {
-            offset: volume_meta.voxel_buffers.push(&render_device),
-        };
+        if volume_meta.voxel_buffers.get(&entity).is_none() {
+            let buffer = render_device.create_buffer(&BufferDescriptor {
+                label: None,
+                size: GpuVoxelBuffer::std140_size_static() as u64,
+                usage: BufferUsages::STORAGE,
+                mapped_at_creation: false,
+            });
+            // TODO: clear unused buffers.
+            volume_meta.voxel_buffers.insert(entity, buffer);
+        }
 
         let voxel_texture = texture_cache.get(
             &render_device,
@@ -502,7 +366,6 @@ pub fn prepare_volumes(
 
             commands.entity(view).insert_bundle((
                 volume_uniform_offset.clone(),
-                voxel_buffer_offset.clone(),
                 VolumeColorAttachment {
                     texture: color_texture,
                 },
@@ -510,38 +373,9 @@ pub fn prepare_volumes(
             ));
         }
 
-        let irradiance_depth_texture = texture_cache.get(
-            &render_device,
-            TextureDescriptor {
-                label: Some("volume_overlay_depth_texture"),
-                size: overlay.irradiance_size,
-                mip_level_count: 1,
-                sample_count: msaa.samples,
-                dimension: TextureDimension::D2,
-                format: TextureFormat::Depth32Float,
-                usage: TextureUsages::RENDER_ATTACHMENT | TextureUsages::TEXTURE_BINDING,
-            },
-        );
-
-        let albedo_depth_texture = texture_cache.get(
-            &render_device,
-            TextureDescriptor {
-                label: Some("volume_overlay_depth_texture"),
-                size: overlay.albedo_size,
-                mip_level_count: 1,
-                sample_count: msaa.samples,
-                dimension: TextureDimension::D2,
-                format: TextureFormat::Depth32Float,
-                usage: TextureUsages::RENDER_ATTACHMENT | TextureUsages::TEXTURE_BINDING,
-            },
-        );
-
         commands.entity(entity).insert_bundle((
             volume_uniform_offset.clone(),
-            voxel_buffer_offset.clone(),
             VolumeBindings {
-                irradiance_depth_texture,
-                albedo_depth_texture,
                 voxel_texture,
                 anisotropic_textures,
                 texture_sampler: texture_sampler.clone(),
