@@ -18,6 +18,7 @@ use bevy::{
         },
         render_resource::*,
         renderer::RenderDevice,
+        texture::{BevyDefault, CachedTexture, TextureCache},
         view::{ExtractedView, ViewDepthTexture, ViewTarget},
         RenderApp, RenderStage,
     },
@@ -29,19 +30,192 @@ impl Plugin for OverlayPlugin {
     fn build(&self, app: &mut App) {
         app.add_asset::<OverlayMaterial>()
             .add_plugin(ExtractComponentPlugin::<Handle<OverlayMaterial>>::default())
-            .add_plugin(RenderAssetPlugin::<OverlayMaterial>::default());
+            .add_plugin(RenderAssetPlugin::<OverlayMaterial>::default())
+            .init_resource::<ScreenOverlay>()
+            .add_startup_system(setup);
+
         if let Ok(render_app) = app.get_sub_app_mut(RenderApp) {
             render_app
                 .init_resource::<DrawFunctions<Overlay>>()
                 .init_resource::<MaterialPipeline<OverlayMaterial>>()
                 .init_resource::<SpecializedPipelines<MaterialPipeline<OverlayMaterial>>>()
                 .add_render_command::<Overlay, DrawMaterial<OverlayMaterial>>()
+                .add_system_to_stage(RenderStage::Extract, extract_screen_overlay)
+                .add_system_to_stage(RenderStage::Prepare, prepare_screen_overlay)
                 .add_system_to_stage(RenderStage::Prepare, prepare_overlay_phase)
                 .add_system_to_stage(RenderStage::Queue, queue_material_meshes);
         }
     }
 }
 
+fn setup(
+    mut commands: Commands,
+    overlay: Res<ScreenOverlay>,
+    mut meshes: ResMut<Assets<Mesh>>,
+    mut materials: ResMut<Assets<OverlayMaterial>>,
+) {
+    let irradiance_resolve = overlay.irradiance_resolve.clone();
+    let albedo_resolve = overlay.albedo_resolve.clone();
+
+    commands.spawn_bundle(MaterialMeshBundle {
+        mesh: meshes.add(shape::Quad::new(Vec2::ZERO).into()),
+        material: materials.add(OverlayMaterial {
+            irradiance_image: irradiance_resolve,
+            albedo_image: albedo_resolve,
+        }),
+        ..Default::default()
+    });
+}
+
+fn extract_screen_overlay(mut commands: Commands, screen_overlay: Res<ScreenOverlay>) {
+    commands.insert_resource(screen_overlay.clone());
+}
+
+fn prepare_screen_overlay(
+    mut commands: Commands,
+    images: Res<RenderAssets<Image>>,
+    overlay: Res<ScreenOverlay>,
+    msaa: Res<Msaa>,
+    render_device: Res<RenderDevice>,
+    mut texture_cache: ResMut<TextureCache>,
+) {
+    let irradiance_depth = texture_cache.get(
+        &render_device,
+        TextureDescriptor {
+            label: Some("volume_overlay_depth_texture"),
+            size: overlay.irradiance_size,
+            mip_level_count: 1,
+            sample_count: msaa.samples,
+            dimension: TextureDimension::D2,
+            format: TextureFormat::Depth32Float,
+            usage: TextureUsages::RENDER_ATTACHMENT | TextureUsages::TEXTURE_BINDING,
+        },
+    );
+
+    let albedo_depth = texture_cache.get(
+        &render_device,
+        TextureDescriptor {
+            label: Some("volume_overlay_depth_texture"),
+            size: overlay.albedo_size,
+            mip_level_count: 1,
+            sample_count: msaa.samples,
+            dimension: TextureDimension::D2,
+            format: TextureFormat::Depth32Float,
+            usage: TextureUsages::RENDER_ATTACHMENT | TextureUsages::TEXTURE_BINDING,
+        },
+    );
+
+    let retrieve_textures = || {
+        let irradiance = images.get(&overlay.irradiance)?.texture_view.clone();
+        let irradiance_resolve = images
+            .get(&overlay.irradiance_resolve)?
+            .texture_view
+            .clone();
+        let albedo = images.get(&overlay.albedo)?.texture_view.clone();
+        let albedo_resolve = images.get(&overlay.albedo_resolve)?.texture_view.clone();
+        Some((irradiance, irradiance_resolve, albedo, albedo_resolve))
+    };
+
+    if let Some((irradiance, irradiance_resolve, albedo, albedo_resolve)) = retrieve_textures() {
+        commands.insert_resource(GpuScreenOverlay {
+            irradiance,
+            irradiance_resolve,
+            irradiance_depth,
+            albedo,
+            albedo_resolve,
+            albedo_depth,
+        });
+    }
+}
+
+#[derive(Debug, Clone)]
+pub struct ScreenOverlay {
+    pub irradiance_size: Extent3d,
+    pub irradiance: Handle<Image>,
+    pub irradiance_resolve: Handle<Image>,
+
+    pub albedo_size: Extent3d,
+    pub albedo: Handle<Image>,
+    pub albedo_resolve: Handle<Image>,
+}
+
+pub struct GpuScreenOverlay {
+    pub irradiance: TextureView,
+    pub irradiance_resolve: TextureView,
+    pub irradiance_depth: CachedTexture,
+
+    pub albedo: TextureView,
+    pub albedo_resolve: TextureView,
+    pub albedo_depth: CachedTexture,
+}
+
+impl FromWorld for ScreenOverlay {
+    fn from_world(world: &mut World) -> Self {
+        let windows = world.get_resource::<Windows>().unwrap();
+        let window = windows.get_primary().unwrap();
+        let width = window.width() as u32;
+        let height = window.height() as u32;
+
+        let msaa = world.get_resource::<Msaa>().unwrap();
+        let samples = msaa.samples;
+
+        let mut images = world.get_resource_mut::<Assets<Image>>().unwrap();
+
+        let irradiance_size = Extent3d {
+            width: width >> 1,
+            height: height >> 1,
+            depth_or_array_layers: 1,
+        };
+        let mut image = Image::new_fill(
+            irradiance_size,
+            TextureDimension::D2,
+            &[0, 0, 0, 255],
+            TextureFormat::bevy_default(),
+        );
+        image.texture_descriptor.usage = TextureUsages::COPY_DST
+            | TextureUsages::RENDER_ATTACHMENT
+            | TextureUsages::TEXTURE_BINDING;
+
+        image.texture_descriptor.sample_count = samples;
+        let irradiance = images.add(image.clone());
+
+        image.texture_descriptor.sample_count = 1;
+        let irradiance_resolve = images.add(image);
+
+        let albedo_size = Extent3d {
+            width,
+            height,
+            depth_or_array_layers: 1,
+        };
+        let mut image = Image::new_fill(
+            albedo_size,
+            TextureDimension::D2,
+            &[0, 0, 0, 255],
+            TextureFormat::bevy_default(),
+        );
+        image.texture_descriptor.usage = TextureUsages::COPY_DST
+            | TextureUsages::RENDER_ATTACHMENT
+            | TextureUsages::TEXTURE_BINDING;
+        image.sampler_descriptor.mag_filter = FilterMode::Linear;
+        image.sampler_descriptor.min_filter = FilterMode::Linear;
+
+        image.texture_descriptor.sample_count = samples;
+        let albedo = images.add(image.clone());
+
+        image.texture_descriptor.sample_count = 1;
+        let albedo_resolve = images.add(image);
+
+        Self {
+            irradiance_size,
+            irradiance,
+            irradiance_resolve,
+            albedo_size,
+            albedo,
+            albedo_resolve,
+        }
+    }
+}
+
 #[derive(Debug, Clone, TypeUuid)]
 #[uuid = "3eb25222-95fd-11ec-b909-0242ac120002"]
 pub struct OverlayMaterial {
@@ -187,7 +361,7 @@ impl SpecializedMaterial for OverlayMaterial {
     }
 }
 
-pub fn prepare_overlay_phase(mut commands: Commands, views: Query<Entity, With<Volume>>) {
+fn prepare_overlay_phase(mut commands: Commands, views: Query<Entity, With<Volume>>) {
     for view in views.iter() {
         commands
             .entity(view)
@@ -196,7 +370,7 @@ pub fn prepare_overlay_phase(mut commands: Commands, views: Query<Entity, With<V
 }
 
 #[allow(clippy::too_many_arguments)]
-pub fn queue_material_meshes(
+fn queue_material_meshes(
     draw_functions: Res<DrawFunctions<Overlay>>,
     material_pipeline: Res<MaterialPipeline<OverlayMaterial>>,
     mut pipelines: ResMut<SpecializedPipelines<MaterialPipeline<OverlayMaterial>>>,
@@ -327,7 +501,7 @@ impl render_graph::Node for OverlayPassNode {
         };
 
         let pass_descriptor = RenderPassDescriptor {
-            label: Some("tracing_pass"),
+            label: Some("overlay_pass"),
             color_attachments: &[target.get_color_attachment(Operations {
                 load: LoadOp::Load,
                 store: true,
