@@ -1,6 +1,6 @@
 use crate::{
-    GpuVolume, NotGiReceiver, Volume, VolumeBindings, VolumeMeta, VolumeOverlay,
-    VolumeUniformOffset, VoxelConeTracingSystems, TRACING_SHADER_HANDLE,
+    overlay::GpuScreenOverlay, GpuVolume, NotGiReceiver, Volume, VolumeBindings, VolumeMeta,
+    VolumeUniformOffset, TRACING_SHADER_HANDLE,
 };
 use bevy::{
     core::FloatOrd,
@@ -19,7 +19,10 @@ use bevy::{
             DrawFunctions, EntityPhaseItem, EntityRenderCommand, PhaseItem, RenderCommandResult,
             RenderPhase, SetItemPipeline, TrackedRenderPass,
         },
-        render_resource::{std140::AsStd140, *},
+        render_resource::{
+            std140::{AsStd140, Std140},
+            *,
+        },
         renderer::RenderDevice,
         view::{ExtractedView, VisibleEntities},
         RenderApp, RenderStage,
@@ -32,21 +35,15 @@ impl Plugin for TracingPlugin {
     fn build(&self, app: &mut App) {
         if let Ok(render_app) = app.get_sub_app_mut(RenderApp) {
             render_app
+                .init_resource::<DirectionsUniform>()
                 .init_resource::<TracingPipeline>()
                 .init_resource::<SpecializedPipelines<TracingPipeline>>()
                 .init_resource::<DrawFunctions<Tracing<Opaque3d>>>()
                 .init_resource::<DrawFunctions<Tracing<AlphaMask3d>>>()
                 .init_resource::<DrawFunctions<Tracing<Transparent3d>>>()
                 .init_resource::<DrawFunctions<AmbientOcclusion>>()
-                .add_system_to_stage(
-                    RenderStage::Extract,
-                    extract_receiver.label(VoxelConeTracingSystems::ExtractReceiverFilter),
-                )
-                .add_system_to_stage(
-                    RenderStage::Queue,
-                    queue_tracing_bind_groups
-                        .label(VoxelConeTracingSystems::QueueTracingBindGroups),
-                )
+                .add_system_to_stage(RenderStage::Extract, extract_receiver)
+                .add_system_to_stage(RenderStage::Queue, queue_tracing_bind_groups)
                 .add_system_to_stage(
                     RenderStage::PhaseSort,
                     sort_phase_system::<Tracing<Opaque3d>>,
@@ -78,10 +75,7 @@ impl<M: SpecializedMaterial> Plugin for TracingMaterialPlugin<M> {
                 .add_render_command::<Tracing<AlphaMask3d>, DrawTracingMesh<M>>()
                 .add_render_command::<Tracing<Transparent3d>, DrawTracingMesh<M>>()
                 .add_render_command::<AmbientOcclusion, DrawTracingMesh<M>>()
-                .add_system_to_stage(
-                    RenderStage::Queue,
-                    queue_tracing_meshes::<M>.label(VoxelConeTracingSystems::QueueTracing),
-                );
+                .add_system_to_stage(RenderStage::Queue, queue_tracing_meshes::<M>);
         }
     }
 }
@@ -114,6 +108,16 @@ impl FromWorld for TracingPipeline {
             BindGroupLayoutEntry {
                 binding: 6,
                 visibility: ShaderStages::FRAGMENT,
+                ty: BindingType::Buffer {
+                    ty: BufferBindingType::Uniform,
+                    has_dynamic_offset: false,
+                    min_binding_size: BufferSize::new(GpuDirections::std140_size_static() as u64),
+                },
+                count: None,
+            },
+            BindGroupLayoutEntry {
+                binding: 7,
+                visibility: ShaderStages::FRAGMENT,
                 ty: BindingType::Buffer {
                     ty: BufferBindingType::Uniform,
                     has_dynamic_offset: true,
@@ -122,7 +126,7 @@ impl FromWorld for TracingPipeline {
                 count: None,
             },
             BindGroupLayoutEntry {
-                binding: 7,
+                binding: 8,
                 visibility: ShaderStages::FRAGMENT,
                 ty: BindingType::Texture {
                     sample_type: TextureSampleType::Float { filterable: true },
@@ -132,7 +136,7 @@ impl FromWorld for TracingPipeline {
                 count: None,
             },
             BindGroupLayoutEntry {
-                binding: 8,
+                binding: 9,
                 visibility: ShaderStages::FRAGMENT,
                 ty: BindingType::Sampler(SamplerBindingType::Filtering),
                 count: None,
@@ -201,12 +205,54 @@ impl SpecializedPipeline for TracingPipeline {
     }
 }
 
+pub struct DirectionsUniform(pub Buffer);
+
+impl FromWorld for DirectionsUniform {
+    fn from_world(world: &mut World) -> Self {
+        let render_device = world.get_resource::<RenderDevice>().unwrap();
+        let buffer = render_device.create_buffer_with_data(&BufferInitDescriptor {
+            label: None,
+            contents: GpuDirections::default().as_std140().as_bytes(),
+            usage: BufferUsages::UNIFORM,
+        });
+        Self(buffer)
+    }
+}
+
+#[derive(AsStd140)]
+struct GpuDirections {
+    data: [Vec3; 14],
+}
+
+impl Default for GpuDirections {
+    fn default() -> Self {
+        Self {
+            data: [
+                Vec3::new(1.0, 1.0, 1.0),
+                Vec3::new(-1.0, 1.0, 1.0),
+                Vec3::new(1.0, -1.0, 1.0),
+                Vec3::new(-1.0, -1.0, 1.0),
+                Vec3::new(1.0, 1.0, -1.0),
+                Vec3::new(-1.0, 1.0, -1.0),
+                Vec3::new(1.0, -1.0, -1.0),
+                Vec3::new(-1.0, -1.0, -1.0),
+                Vec3::new(1.0, 0.0, 0.0),
+                Vec3::new(0.0, 1.0, 0.0),
+                Vec3::new(0.0, 0.0, 1.0),
+                Vec3::new(-1.0, 0.0, 0.0),
+                Vec3::new(0.0, -1.0, 0.0),
+                Vec3::new(0.0, 0.0, -1.0),
+            ],
+        }
+    }
+}
+
 #[derive(Component)]
 pub struct TracingBindGroup {
     value: BindGroup,
 }
 
-pub fn extract_receiver(
+fn extract_receiver(
     mut commands: Commands,
     query: Query<(Entity, &Handle<Mesh>), With<NotGiReceiver>>,
 ) {
@@ -217,7 +263,7 @@ pub fn extract_receiver(
 
 #[allow(clippy::too_many_arguments)]
 #[allow(clippy::type_complexity)]
-pub fn queue_tracing_meshes<M: SpecializedMaterial>(
+fn queue_tracing_meshes<M: SpecializedMaterial>(
     opaque_draw_functions: Res<DrawFunctions<Tracing<Opaque3d>>>,
     alpha_mask_draw_functions: Res<DrawFunctions<Tracing<AlphaMask3d>>>,
     transparent_draw_functions: Res<DrawFunctions<Tracing<Transparent3d>>>,
@@ -351,6 +397,7 @@ pub fn queue_tracing_bind_groups(
     mut commands: Commands,
     render_device: Res<RenderDevice>,
     tracing_pipeline: Res<TracingPipeline>,
+    directions: Res<DirectionsUniform>,
     volume_meta: Res<VolumeMeta>,
     volume_query: Query<(Entity, &VolumeBindings), With<Volume>>,
 ) {
@@ -365,14 +412,22 @@ pub fn queue_tracing_bind_groups(
         let tracing_bindings = vec![
             BindGroupEntry {
                 binding: 6,
-                resource: volume_meta.volume_uniforms.binding().unwrap(),
+                resource: BindingResource::Buffer(BufferBinding {
+                    buffer: &directions.0,
+                    offset: 0,
+                    size: None,
+                }),
             },
             BindGroupEntry {
                 binding: 7,
-                resource: BindingResource::TextureView(&volume_bindings.voxel_texture.default_view),
+                resource: volume_meta.volume_uniforms.binding().unwrap(),
             },
             BindGroupEntry {
                 binding: 8,
+                resource: BindingResource::TextureView(&volume_bindings.voxel_texture.default_view),
+            },
+            BindGroupEntry {
+                binding: 9,
                 resource: BindingResource::Sampler(&volume_bindings.texture_sampler),
             },
         ];
@@ -492,8 +547,6 @@ pub struct TracingPassNode {
             &'static RenderPhase<Tracing<AlphaMask3d>>,
             &'static RenderPhase<Tracing<Transparent3d>>,
             &'static RenderPhase<AmbientOcclusion>,
-            &'static VolumeOverlay,
-            &'static VolumeBindings,
         ),
         With<ExtractedView>,
     >,
@@ -525,35 +578,30 @@ impl render_graph::Node for TracingPassNode {
         world: &World,
     ) -> Result<(), render_graph::NodeRunError> {
         let view_entity = graph.get_input_entity(Self::IN_VIEW)?;
-        let (
-            opaque_phase,
-            alpha_mask_phase,
-            transparent_phase,
-            ambient_occlusion_phase,
-            overlay,
-            bindings,
-        ) = match self.query.get_manual(world, view_entity) {
-            Ok(query) => query,
-            Err(_) => return Ok(()),
-        };
+        let (opaque_phase, alpha_mask_phase, transparent_phase, ambient_occlusion_phase) =
+            match self.query.get_manual(world, view_entity) {
+                Ok(query) => query,
+                Err(_) => return Ok(()),
+            };
 
-        let images = world.get_resource::<RenderAssets<Image>>().unwrap();
-        let view = &images[&overlay.irradiance].texture_view;
-        let resolve_target = &images[&overlay.irradiance_resolve].texture_view;
+        let overlay = match world.get_resource::<GpuScreenOverlay>() {
+            Some(overlay) => overlay,
+            None => return Ok(()),
+        };
 
         {
             let pass_descriptor = RenderPassDescriptor {
                 label: Some("tracing_opaque_pass"),
                 color_attachments: &[RenderPassColorAttachment {
-                    view,
-                    resolve_target: Some(resolve_target),
+                    view: &overlay.irradiance,
+                    resolve_target: Some(&overlay.irradiance_resolve),
                     ops: Operations {
                         load: LoadOp::Clear(Color::NONE.into()),
                         store: true,
                     },
                 }],
                 depth_stencil_attachment: Some(RenderPassDepthStencilAttachment {
-                    view: &bindings.irradiance_depth_texture.default_view,
+                    view: &overlay.irradiance_depth.default_view,
                     depth_ops: Some(Operations {
                         load: LoadOp::Clear(0.0),
                         store: true,
@@ -582,15 +630,15 @@ impl render_graph::Node for TracingPassNode {
             let pass_descriptor = RenderPassDescriptor {
                 label: Some("tracing_alpha_mask_pass"),
                 color_attachments: &[RenderPassColorAttachment {
-                    view,
-                    resolve_target: Some(resolve_target),
+                    view: &overlay.irradiance,
+                    resolve_target: Some(&overlay.irradiance_resolve),
                     ops: Operations {
                         load: LoadOp::Load,
                         store: true,
                     },
                 }],
                 depth_stencil_attachment: Some(RenderPassDepthStencilAttachment {
-                    view: &bindings.irradiance_depth_texture.default_view,
+                    view: &overlay.irradiance_depth.default_view,
                     depth_ops: Some(Operations {
                         load: LoadOp::Load,
                         store: true,
@@ -619,15 +667,15 @@ impl render_graph::Node for TracingPassNode {
             let pass_descriptor = RenderPassDescriptor {
                 label: Some("tracing_transparent_pass"),
                 color_attachments: &[RenderPassColorAttachment {
-                    view,
-                    resolve_target: Some(resolve_target),
+                    view: &overlay.irradiance,
+                    resolve_target: Some(&overlay.irradiance_resolve),
                     ops: Operations {
                         load: LoadOp::Load,
                         store: true,
                     },
                 }],
                 depth_stencil_attachment: Some(RenderPassDepthStencilAttachment {
-                    view: &bindings.irradiance_depth_texture.default_view,
+                    view: &overlay.irradiance_depth.default_view,
                     depth_ops: Some(Operations {
                         load: LoadOp::Load,
                         store: false,
@@ -656,15 +704,15 @@ impl render_graph::Node for TracingPassNode {
             let pass_descriptor = RenderPassDescriptor {
                 label: Some("tracing_ambient_occlusion_pass"),
                 color_attachments: &[RenderPassColorAttachment {
-                    view,
-                    resolve_target: Some(resolve_target),
+                    view: &overlay.irradiance,
+                    resolve_target: Some(&overlay.irradiance_resolve),
                     ops: Operations {
                         load: LoadOp::Load,
                         store: true,
                     },
                 }],
                 depth_stencil_attachment: Some(RenderPassDepthStencilAttachment {
-                    view: &bindings.irradiance_depth_texture.default_view,
+                    view: &overlay.irradiance_depth.default_view,
                     depth_ops: Some(Operations {
                         load: LoadOp::Clear(0.0),
                         store: true,
