@@ -1,9 +1,8 @@
-use super::{
-    GpuVolume, Volume, VolumeBindings, VolumeColorAttachment, VolumeMeta, VolumeUniformOffset,
-    VolumeView, VoxelConeTracingSystems, VOXEL_ANISOTROPIC_MIPMAP_LEVEL_COUNT, VOXEL_SHADER_HANDLE,
+use crate::{
+    GpuVolume, GpuVoxelBuffer, NotGiCaster, Volume, VolumeBindings, VolumeColorAttachment,
+    VolumeMeta, VolumeUniformOffset, VOXEL_ANISOTROPIC_MIPMAP_LEVEL_COUNT, VOXEL_SHADER_HANDLE,
     VOXEL_SIZE,
 };
-use crate::{GpuVoxelBuffer, NotGiCaster, VoxelBufferOffset};
 use bevy::{
     core::FloatOrd,
     ecs::system::{
@@ -37,19 +36,18 @@ use std::{borrow::Cow, f32::consts::FRAC_PI_2, marker::PhantomData, num::NonZero
 pub struct VoxelPlugin;
 impl Plugin for VoxelPlugin {
     fn build(&self, app: &mut App) {
+        app.add_system_to_stage(CoreStage::PostUpdate, add_volume_views.exclusive_system())
+            .add_system_to_stage(CoreStage::PostUpdate, check_visibility);
+
         if let Ok(render_app) = app.get_sub_app_mut(RenderApp) {
             render_app
                 .init_resource::<VoxelPipeline>()
                 .init_resource::<SpecializedPipelines<VoxelPipeline>>()
                 .init_resource::<DrawFunctions<Voxel>>()
-                .add_system_to_stage(
-                    RenderStage::Queue,
-                    queue_voxel_bind_groups.label(VoxelConeTracingSystems::QueueVoxelBindGroups),
-                )
-                .add_system_to_stage(
-                    RenderStage::Queue,
-                    queue_mipmap_bind_groups.label(VoxelConeTracingSystems::QueueMipmapBindGroups),
-                );
+                .add_system_to_stage(RenderStage::Extract, extract_views)
+                .add_system_to_stage(RenderStage::Queue, queue_volume_view_bind_groups)
+                .add_system_to_stage(RenderStage::Queue, queue_voxel_bind_groups)
+                .add_system_to_stage(RenderStage::Queue, queue_mipmap_bind_groups);
         }
     }
 }
@@ -62,14 +60,14 @@ impl<M: SpecializedMaterial> Plugin for VoxelMaterialPlugin<M> {
         if let Ok(render_app) = app.get_sub_app_mut(RenderApp) {
             render_app
                 .add_render_command::<Voxel, DrawVoxelMesh<M>>()
-                .add_system_to_stage(
-                    RenderStage::Queue,
-                    queue_voxel_meshes::<M>.label(VoxelConeTracingSystems::QueueVoxel),
-                );
+                .add_system_to_stage(RenderStage::Queue, queue_voxel_meshes::<M>);
         }
     }
 }
 
+#[derive(Component)]
+pub struct VolumeView;
+
 #[derive(Component)]
 pub struct VoxelBindGroup {
     pub value: BindGroup,
@@ -128,7 +126,7 @@ impl FromWorld for VoxelPipeline {
                     visibility: ShaderStages::FRAGMENT,
                     ty: BindingType::Buffer {
                         ty: BufferBindingType::Storage { read_only: false },
-                        has_dynamic_offset: true,
+                        has_dynamic_offset: false,
                         min_binding_size: BufferSize::new(
                             GpuVoxelBuffer::std140_size_static() as u64
                         ),
@@ -166,7 +164,7 @@ impl FromWorld for VoxelPipeline {
                     visibility: ShaderStages::COMPUTE,
                     ty: BindingType::Buffer {
                         ty: BufferBindingType::Storage { read_only: false },
-                        has_dynamic_offset: true,
+                        has_dynamic_offset: false,
                         min_binding_size: BufferSize::new(
                             GpuVoxelBuffer::std140_size_static() as u64
                         ),
@@ -252,7 +250,7 @@ impl SpecializedPipeline for VoxelPipeline {
     }
 }
 
-pub fn add_volume_views(mut commands: Commands, mut volumes: Query<&mut Volume>) {
+fn add_volume_views(mut commands: Commands, mut volumes: Query<&mut Volume>) {
     for mut volume in volumes.iter_mut() {
         if !volume.views.is_empty() {
             continue;
@@ -294,7 +292,7 @@ pub fn add_volume_views(mut commands: Commands, mut volumes: Query<&mut Volume>)
 }
 
 #[allow(clippy::type_complexity)]
-pub fn check_visibility(
+fn check_visibility(
     mut view_query: Query<
         (&mut VisibleEntities, &Frustum, Option<&RenderLayers>),
         With<VolumeView>,
@@ -357,7 +355,7 @@ pub fn check_visibility(
     }
 }
 
-pub fn extract_views(
+fn extract_views(
     mut commands: Commands,
     query: Query<
         (
@@ -386,7 +384,7 @@ pub fn extract_views(
 }
 
 #[allow(clippy::too_many_arguments)]
-pub fn queue_volume_view_bind_groups(
+fn queue_volume_view_bind_groups(
     mut commands: Commands,
     render_device: Res<RenderDevice>,
     mesh_pipeline: Res<MeshPipeline>,
@@ -478,14 +476,14 @@ pub fn queue_volume_view_bind_groups(
     }
 }
 
-pub fn queue_voxel_bind_groups(
+fn queue_voxel_bind_groups(
     mut commands: Commands,
     render_device: Res<RenderDevice>,
     voxel_pipeline: Res<VoxelPipeline>,
     volume_meta: Res<VolumeMeta>,
-    volumes: Query<(&Volume, &VolumeBindings)>,
+    volumes: Query<(Entity, &Volume, &VolumeBindings)>,
 ) {
-    for (volume, bindings) in volumes.iter() {
+    for (entity, volume, bindings) in volumes.iter() {
         for view in volume.views.iter().cloned() {
             let bind_group = render_device.create_bind_group(&BindGroupDescriptor {
                 label: Some("voxel_bind_group"),
@@ -503,7 +501,11 @@ pub fn queue_voxel_bind_groups(
                     },
                     BindGroupEntry {
                         binding: 2,
-                        resource: volume_meta.voxel_buffers.binding().unwrap(),
+                        resource: BindingResource::Buffer(BufferBinding {
+                            buffer: volume_meta.voxel_buffers.get(&entity).unwrap(),
+                            offset: 0,
+                            size: None,
+                        }),
                     },
                 ],
             });
@@ -613,7 +615,11 @@ pub fn queue_mipmap_bind_groups(
                             },
                             BindGroupEntry {
                                 binding: 2,
-                                resource: volume_meta.voxel_buffers.binding().unwrap(),
+                                resource: BindingResource::Buffer(BufferBinding {
+                                    buffer: volume_meta.voxel_buffers.get(&entity).unwrap(),
+                                    offset: 0,
+                                    size: None,
+                                }),
                             },
                         ],
                     });
@@ -642,7 +648,11 @@ pub fn queue_mipmap_bind_groups(
                 },
                 BindGroupEntry {
                     binding: 2,
-                    resource: volume_meta.voxel_buffers.binding().unwrap(),
+                    resource: BindingResource::Buffer(BufferBinding {
+                        buffer: volume_meta.voxel_buffers.get(&entity).unwrap(),
+                        offset: 0,
+                        size: None,
+                    }),
                 },
             ],
         });
@@ -695,11 +705,7 @@ pub type DrawVoxelMesh<M> = (
 
 pub struct SetVoxelBindGroup<const I: usize>;
 impl<const I: usize> EntityRenderCommand for SetVoxelBindGroup<I> {
-    type Param = SQuery<(
-        Read<VolumeUniformOffset>,
-        Read<VoxelBufferOffset>,
-        Read<VoxelBindGroup>,
-    )>;
+    type Param = SQuery<(Read<VolumeUniformOffset>, Read<VoxelBindGroup>)>;
 
     fn render<'w>(
         view: Entity,
@@ -707,12 +713,8 @@ impl<const I: usize> EntityRenderCommand for SetVoxelBindGroup<I> {
         query: SystemParamItem<'w, '_, Self::Param>,
         pass: &mut TrackedRenderPass<'w>,
     ) -> RenderCommandResult {
-        let (volume_uniform_offset, voxel_buffer_offset, bind_group) = query.get(view).unwrap();
-        pass.set_bind_group(
-            I,
-            &bind_group.value,
-            &[volume_uniform_offset.offset, voxel_buffer_offset.offset],
-        );
+        let (volume_uniform_offset, bind_group) = query.get(view).unwrap();
+        pass.set_bind_group(I, &bind_group.value, &[volume_uniform_offset.offset]);
         RenderCommandResult::Success
     }
 }
@@ -787,7 +789,7 @@ impl render_graph::Node for VoxelPassNode {
 }
 
 pub struct MipmapPassNode {
-    query: QueryState<(&'static MipmapBindGroup, &'static VoxelBufferOffset), With<Volume>>,
+    query: QueryState<&'static MipmapBindGroup, With<Volume>>,
 }
 
 impl MipmapPassNode {
@@ -815,18 +817,18 @@ impl render_graph::Node for MipmapPassNode {
             .command_encoder
             .begin_compute_pass(&ComputePassDescriptor::default());
 
-        for (mipmap_bind_group, voxel_buffer_offset) in self.query.iter_manual(world) {
-            let count = (VOXEL_SIZE / 4) as u32;
+        for mipmap_bind_group in self.query.iter_manual(world) {
+            let count = (VOXEL_SIZE / 8) as u32;
             pass.set_pipeline(&pipeline.fill_pipeline);
-            pass.set_bind_group(0, &mipmap_bind_group.clear, &[voxel_buffer_offset.offset]);
+            pass.set_bind_group(0, &mipmap_bind_group.clear, &[]);
             pass.dispatch(count, count, count);
 
             for (level, bind_groups) in mipmap_bind_group.mipmaps.iter().enumerate() {
                 for direction in 0..6 {
                     let size = (VOXEL_SIZE / (2 << level)) as u32;
-                    let count = (size / 4).max(1);
+                    let count = (size / 8).max(1);
                     pass.set_pipeline(&pipeline.mipmap_pipelines[direction]);
-                    pass.set_bind_group(0, &bind_groups[direction], &[voxel_buffer_offset.offset]);
+                    pass.set_bind_group(0, &bind_groups[direction], &[]);
                     pass.dispatch(count, count, count);
                 }
             }
@@ -837,7 +839,7 @@ impl render_graph::Node for MipmapPassNode {
 }
 
 pub struct VoxelClearPassNode {
-    query: QueryState<(&'static MipmapBindGroup, &'static VoxelBufferOffset), With<Volume>>,
+    query: QueryState<&'static MipmapBindGroup, With<Volume>>,
 }
 
 impl VoxelClearPassNode {
@@ -864,10 +866,10 @@ impl render_graph::Node for VoxelClearPassNode {
             .command_encoder
             .begin_compute_pass(&ComputePassDescriptor::default());
 
-        for (mipmap_bind_group, voxel_buffer_offset) in self.query.iter_manual(world) {
-            let count = (VOXEL_SIZE / 4) as u32;
+        for mipmap_bind_group in self.query.iter_manual(world) {
+            let count = (VOXEL_SIZE / 8) as u32;
             pass.set_pipeline(&pipeline.clear_pipeline);
-            pass.set_bind_group(0, &mipmap_bind_group.clear, &[voxel_buffer_offset.offset]);
+            pass.set_bind_group(0, &mipmap_bind_group.clear, &[]);
             pass.dispatch(count, count, count);
         }
 
